use std::{convert::Infallible, fmt::Debug};

use hex::FromHex;
use secrecy::{ExposeSecret, SecretString, SecretVec};
use serde::{Deserialize, Serialize};

#[derive(Debug, Clone, Deserialize)]
pub struct Password(pub SecretString);
impl From<String> for Password {
    fn from(value: String) -> Self {
        Self(SecretString::new(value))
    }
}

impl PartialEq for Password {
    fn eq(&self, other: &Self) -> bool {
        self.0.expose_secret() == other.0.expose_secret()
    }
}

#[derive(Debug, Clone, Deserialize)]
pub struct UserToken(pub SecretString);
impl From<String> for UserToken {
    fn from(value: String) -> Self {
        Self(SecretString::new(value))
    }
}

impl PartialEq for UserToken {
    fn eq(&self, other: &Self) -> bool {
        self.0.expose_secret() == other.0.expose_secret()
    }
}

#[derive(Debug, Clone, Deserialize)]
pub struct UserServerKey(pub SecretString);
impl From<String> for UserServerKey {
    fn from(value: String) -> Self {
        Self(SecretString::new(value))
    }
}

#[derive(Debug, Clone, Deserialize)]
pub struct ServerPadlock(pub SecretString);
impl From<String> for ServerPadlock {
    fn from(value: String) -> Self {
        Self(SecretString::new(value))
    }
}

/// Not actually a hash, just a random value handed out to servers on request.
#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct ServerHash(pub String);

#[derive(Deserialize)]
pub struct PadlockGenerationSecret(pub SecretVec<u8>);
impl Debug for PadlockGenerationSecret {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        f.debug_tuple("PadlockGenerationSecret")
            .field(&"[REDACTED Vec<u8>]")
            .finish()
    }
}
impl Clone for PadlockGenerationSecret {
    fn clone(&self) -> Self {
        PadlockGenerationSecret(self.0.expose_secret().clone().into())
    }
}

impl FromHex for PadlockGenerationSecret {
    type Error = Infallible;

    fn from_hex<T: AsRef<[u8]>>(hex: T) -> Result<Self, Self::Error> {
        Ok(Self(hex.as_ref().to_vec().into()))
    }
}
